@@ -1,5 +1,5 @@
 /*
-    $Id: freebsd.c,v 1.14 2005/01/27 04:54:27 rocky Exp $
+    $Id: freebsd.c,v 1.30 2005/04/14 01:24:32 rocky Exp $
 
     Copyright (C) 2003, 2004, 2005 Rocky Bernstein <rocky@panix.com>
 
@@ -27,7 +27,7 @@
 # include "config.h"
 #endif
 
-static const char _rcsid[] = "$Id: freebsd.c,v 1.14 2005/01/27 04:54:27 rocky Exp $";
+static const char _rcsid[] = "$Id: freebsd.c,v 1.30 2005/04/14 01:24:32 rocky Exp $";
 
 #include "freebsd.h"
 
@@ -39,6 +39,8 @@ static const char _rcsid[] = "$Id: freebsd.c,v 1.14 2005/01/27 04:54:27 rocky Ex
 
 #include <cdio/sector.h>
 
+static lba_t get_track_lba_freebsd(void *p_user_data, track_t i_track);
+
 static access_mode_t 
 str_to_access_mode_freebsd(const char *psz_access_mode) 
 {
@@ -58,7 +60,7 @@ str_to_access_mode_freebsd(const char *psz_access_mode)
 }
 
 static void
-_free_freebsd (void *p_obj)
+free_freebsd (void *p_obj)
 {
   _img_private_t *p_env = p_obj;
 
@@ -87,12 +89,12 @@ cdio_is_cdrom(char *drive, char *mnttype)
    Returns 0 if no error. 
  */
 static driver_return_code_t
-_read_audio_sectors_freebsd (void *p_user_data, void *p_buf, lsn_t i_lsn,
+read_audio_sectors_freebsd (void *p_user_data, void *p_buf, lsn_t i_lsn,
 			     unsigned int i_blocks)
 {
   _img_private_t *p_env = p_user_data;
   if ( p_env->access_mode == _AM_CAM ) {
-    return scsi_mmc_read_sectors( p_env->gen.cdio, p_buf, i_lsn,
+    return mmc_read_sectors( p_env->gen.cdio, p_buf, i_lsn,
                                   CDIO_MMC_READ_TYPE_CDDA, i_blocks);
   } else 
     return read_audio_sectors_freebsd_ioctl(p_user_data, p_buf, i_lsn, 
@@ -104,8 +106,8 @@ _read_audio_sectors_freebsd (void *p_user_data, void *p_buf, lsn_t i_lsn,
    from i_lsn. Returns 0 if no error. 
  */
 static driver_return_code_t
-_read_mode2_sector_freebsd (void *p_user_data, void *data, lsn_t i_lsn,
-			    bool b_form2)
+read_mode2_sector_freebsd (void *p_user_data, void *data, lsn_t i_lsn,
+			   bool b_form2)
 {
   _img_private_t *p_env = p_user_data;
 
@@ -116,28 +118,28 @@ _read_mode2_sector_freebsd (void *p_user_data, void *data, lsn_t i_lsn,
 }
 
 /*!
-   Reads nblocks of mode2 sectors from cd device into data starting
+   Reads i_blocks of mode2 sectors from cd device into data starting
    from lsn.
  */
 static driver_return_code_t
-_read_mode2_sectors_freebsd (void *user_data, void *data, lsn_t lsn, 
-			  bool b_form2, unsigned int nblocks)
+read_mode2_sectors_freebsd (void *p_user_data, void *p_data, lsn_t i_lsn,
+			    bool b_form2, unsigned int i_blocks)
 {
-  _img_private_t *env = user_data;
+  _img_private_t *p_env = p_user_data;
 
-  if ( env->access_mode == _AM_CAM  && b_form2) {
+  if ( p_env->access_mode == _AM_CAM  && b_form2 ) {
     /* We have a routine that covers this case without looping. */
-    return read_mode2_sectors_freebsd_cam(env, data, lsn, nblocks);
+    return read_mode2_sectors_freebsd_cam(p_env, p_data, i_lsn, i_blocks);
   } else {
     unsigned int i;
-    unsigned int i_blocksize = b_form2 ? M2RAW_SECTOR_SIZE : CDIO_CD_FRAMESIZE;
+    uint16_t i_blocksize = b_form2 ? M2RAW_SECTOR_SIZE : CDIO_CD_FRAMESIZE;
   
     /* For each frame, pick out the data part we need */
-    for (i = 0; i < nblocks; i++) {
-      int retval = _read_mode2_sector_freebsd (env, 
-					       ((char *)data) + 
+    for (i = 0; i < i_blocks; i++) {
+      int retval = read_mode2_sector_freebsd (p_env, 
+					       ((char *)p_data) + 
 					       (i_blocksize * i),
-					       lsn + i, b_form2);
+					       i_lsn + i, b_form2);
       if (retval) return retval;
     }
   }
@@ -171,7 +173,7 @@ get_disc_last_lsn_freebsd (void *p_obj)
   and nonzero if there as an error.
 */
 static driver_return_code_t
-_set_arg_freebsd (void *p_user_data, const char key[], const char value[])
+set_arg_freebsd (void *p_user_data, const char key[], const char value[])
 {
   _img_private_t *p_env = p_user_data;
 
@@ -260,24 +262,188 @@ read_toc_freebsd (void *p_user_data)
   return true;
 }
 
+/*!
+  Get the volume of an audio CD.
+
+  @param p_cdio the CD object to be acted upon.
+*/
+static driver_return_code_t
+audio_get_volume_freebsd (void *p_user_data,
+			  /*out*/ cdio_audio_volume_t *p_volume)
+{
+
+  const _img_private_t *p_env = p_user_data;
+  return ioctl(p_env->gen.fd, CDIOCGETVOL, p_volume);
+}
+
+/*!
+  Pause playing CD through analog output
+  
+  @param p_cdio the CD object to be acted upon.
+*/
+static driver_return_code_t
+audio_pause_freebsd (void *p_user_data)
+{
+
+  const _img_private_t *p_env = p_user_data;
+  return ioctl(p_env->gen.fd, CDIOCPAUSE);
+}
+
+/*!
+  Playing starting at given MSF through analog output
+  
+  @param p_cdio the CD object to be acted upon.
+*/
+static driver_return_code_t
+audio_play_msf_freebsd (void *p_user_data, msf_t *p_start_msf, 
+			msf_t *p_end_msf)
+{
+  const _img_private_t *p_env = p_user_data;
+  struct ioc_play_msf freebsd_play_msf;
+
+  freebsd_play_msf.start_m = cdio_from_bcd8(p_start_msf->m);
+  freebsd_play_msf.start_s = cdio_from_bcd8(p_start_msf->s);
+  freebsd_play_msf.start_f = cdio_from_bcd8(p_start_msf->f);
+
+  freebsd_play_msf.end_m = cdio_from_bcd8(p_end_msf->m);
+  freebsd_play_msf.end_s = cdio_from_bcd8(p_end_msf->s);
+  freebsd_play_msf.end_f = cdio_from_bcd8(p_end_msf->f);
+
+  return ioctl(p_env->gen.fd, CDIOCPLAYMSF, &freebsd_play_msf);
+}
+
+/*!
+  Playing CD through analog output at the desired track and index
+  
+  @param p_user_data the CD object to be acted upon.
+  @param p_track_index location to start/end.
+*/
+static driver_return_code_t
+audio_play_track_index_freebsd (void *p_user_data, 
+				cdio_track_index_t *p_track_index)
+{
+  const _img_private_t *p_env = p_user_data;
+  msf_t start_msf;
+  msf_t end_msf;
+  struct ioc_play_msf freebsd_play_msf;
+  lsn_t i_lsn = get_track_lba_freebsd(p_user_data, 
+				      p_track_index->i_start_track);
+
+  cdio_lsn_to_msf(i_lsn, &start_msf);
+  i_lsn = get_track_lba_freebsd(p_user_data, p_track_index->i_end_track);
+  cdio_lsn_to_msf(i_lsn, &end_msf);
+
+  freebsd_play_msf.start_m = start_msf.m;
+  freebsd_play_msf.start_s = start_msf.s;
+  freebsd_play_msf.start_f = start_msf.f;
+
+  freebsd_play_msf.end_m = end_msf.m;
+  freebsd_play_msf.end_s = end_msf.s;
+  freebsd_play_msf.end_f = end_msf.f;
+
+  return ioctl(p_env->gen.fd, CDIOCPLAYMSF, &freebsd_play_msf);
+
+}
+
+/*!
+  Read Audio Subchannel information
+  
+  @param p_user_data the CD object to be acted upon.
+  @param p_subchannel returned information
+*/
+#if 1
+static driver_return_code_t
+audio_read_subchannel_freebsd (void *p_user_data, 
+			       /*out*/ cdio_subchannel_t *p_subchannel)
+{
+  const _img_private_t *p_env = p_user_data;
+  int i_rc;
+  struct cd_sub_channel_info bsdinfo;
+  struct ioc_read_subchannel read_subchannel;
+  memset(& bsdinfo, 0, sizeof(struct cd_sub_channel_info));
+  read_subchannel.address_format = CD_MSF_FORMAT;
+  read_subchannel.data_format = CD_CURRENT_POSITION;
+  read_subchannel.track = 0;
+  read_subchannel.data_len = sizeof(struct cd_sub_channel_info);
+  read_subchannel.data = & bsdinfo;
+  i_rc = ioctl(p_env->gen.fd, CDIOCREADSUBCHANNEL, &read_subchannel);
+  if (0 == i_rc) {
+    p_subchannel->audio_status = bsdinfo.header.audio_status;
+    p_subchannel->address      = bsdinfo.what.position.addr_type;
+
+    p_subchannel->control      = bsdinfo.what.position.control;
+    p_subchannel->track        = bsdinfo.what.position.track_number;
+    p_subchannel->index        = bsdinfo.what.position.index_number;
+
+    p_subchannel->abs_addr.m = cdio_from_bcd8 (bsdinfo.what.position.absaddr.msf.minute);
+    p_subchannel->abs_addr.s = cdio_from_bcd8 (bsdinfo.what.position.absaddr.msf.second);
+    p_subchannel->abs_addr.f = cdio_from_bcd8 (bsdinfo.what.position.absaddr.msf.frame);
+    p_subchannel->rel_addr.m = cdio_from_bcd8 (bsdinfo.what.position.reladdr.msf.minute);
+    p_subchannel->rel_addr.s = cdio_from_bcd8 (bsdinfo.what.position.reladdr.msf.second);
+    p_subchannel->rel_addr.f = cdio_from_bcd8 (bsdinfo.what.position.reladdr.msf.frame);
+ }
+  return i_rc;
+}
+#endif
+
+/*!
+  Resume playing an audio CD.
+  
+  @param p_cdio the CD object to be acted upon.
+  
+*/
+static driver_return_code_t
+audio_resume_freebsd (void *p_user_data)
+{
+  const _img_private_t *p_env = p_user_data;
+  return ioctl(p_env->gen.fd, CDIOCRESUME, 0);
+}
+
+/*!
+  Set the volume of an audio CD.
+  
+  @param p_cdio the CD object to be acted upon.
+  
+*/
+static driver_return_code_t
+audio_set_volume_freebsd (void *p_user_data, 
+			  cdio_audio_volume_t *p_volume)
+{
+  const _img_private_t *p_env = p_user_data;
+  return ioctl(p_env->gen.fd, CDIOCSETVOL, p_volume);
+}
+
 /*!
   Eject media. Return 1 if successful, 0 otherwise.
  */
 static int 
-_eject_media_freebsd (void *user_data) 
+eject_media_freebsd (void *p_user_data) 
 {
-  _img_private_t *p_env = user_data;
+  _img_private_t *p_env = p_user_data;
 
   return (p_env->access_mode == _AM_IOCTL) 
     ? eject_media_freebsd_ioctl(p_env) 
     : eject_media_freebsd_cam(p_env);
 }
 
+/*!
+  Stop playing an audio CD.
+  
+  @param p_user_data the CD object to be acted upon.
+  
+*/
+static driver_return_code_t 
+audio_stop_freebsd (void *p_user_data)
+{
+  const _img_private_t *p_env = p_user_data;
+  return ioctl(p_env->gen.fd, CDIOCSTOP);
+}
+
 /*!
   Return the value associated with the key "arg".
 */
 static const char *
-_get_arg_freebsd (void *user_data, const char key[])
+get_arg_freebsd (void *user_data, const char key[])
 {
   _img_private_t *env = user_data;
 
@@ -306,13 +472,12 @@ _get_arg_freebsd (void *user_data, const char key[])
 
  */
 static char *
-_get_mcn_freebsd (const void *p_user_data) {
-
+get_mcn_freebsd (const void *p_user_data) {
   const _img_private_t *p_env = p_user_data;
 
   return (p_env->access_mode == _AM_IOCTL) 
     ? get_mcn_freebsd_ioctl(p_env) 
-    : scsi_mmc_get_mcn(p_env->gen.cdio);
+    : mmc_get_mcn(p_env->gen.cdio);
 
 }
 
@@ -343,15 +508,15 @@ get_drive_cap_freebsd (const void *p_user_data,
   p_buf	        Buffer for data, both sending and receiving
  */
 static driver_return_code_t
-run_scsi_cmd_freebsd( void *p_user_data, unsigned int i_timeout_ms,
-		      unsigned int i_cdb, const scsi_mmc_cdb_t *p_cdb, 
-		      scsi_mmc_direction_t e_direction, 
-		      unsigned int i_buf, /*in/out*/ void *p_buf ) 
+run_mmc_cmd_freebsd( void *p_user_data, unsigned int i_timeout_ms,
+		     unsigned int i_cdb, const scsi_mmc_cdb_t *p_cdb, 
+		     scsi_mmc_direction_t e_direction, 
+		     unsigned int i_buf, /*in/out*/ void *p_buf ) 
 {
   const _img_private_t *p_env = p_user_data;
 
   if (p_env->access_mode == _AM_CAM) 
-    return run_scsi_cmd_freebsd_cam( p_user_data, i_timeout_ms, i_cdb, p_cdb, 
+    return run_mmc_cmd_freebsd_cam( p_user_data, i_timeout_ms, i_cdb, p_cdb, 
 				     e_direction, i_buf, p_buf );
   else 
     return DRIVER_OP_UNSUPPORTED;
@@ -364,7 +529,7 @@ run_scsi_cmd_freebsd( void *p_user_data, unsigned int i_timeout_ms,
   
 */
 static track_format_t
-_get_track_format_freebsd(void *p_user_data, track_t i_track) 
+get_track_format_freebsd(void *p_user_data, track_t i_track) 
 {
   _img_private_t *p_env = p_user_data;
 
@@ -399,7 +564,7 @@ _get_track_format_freebsd(void *p_user_data, track_t i_track)
   FIXME: there's gotta be a better design for this and get_track_format?
 */
 static bool
-_get_track_green_freebsd(void *user_data, track_t i_track) 
+get_track_green_freebsd(void *user_data, track_t i_track) 
 {
   _img_private_t *p_env = user_data;
   
@@ -422,7 +587,7 @@ _get_track_green_freebsd(void *user_data, track_t i_track)
   CDIO_INVALID_LBA is returned if there is no track entry.
 */
 static lba_t
-_get_track_lba_freebsd(void *user_data, track_t i_track)
+get_track_lba_freebsd(void *user_data, track_t i_track)
 {
   _img_private_t *p_env = user_data;
 
@@ -552,6 +717,30 @@ cdio_get_default_device_freebsd()
 #endif /*HAVE_FREEBSD_CDROM*/
 }
 
+/*!
+  Close tray on CD-ROM.
+  
+  @param psz_device the CD-ROM drive to be closed.
+  
+*/
+driver_return_code_t 
+close_tray_freebsd (const char *psz_device)
+{
+#ifdef HAVE_FREEBSD_CDROM
+  int fd = open (psz_device, O_RDONLY|O_NONBLOCK, 0);
+  int i_rc;
+  
+  if((i_rc = ioctl(fd, CDIOCCLOSE)) != 0) {
+    cdio_warn ("ioctl CDIOCCLOSE failed: %s\n", strerror(errno));  
+    return DRIVER_OP_ERROR;
+  }
+  close(fd);
+  return DRIVER_OP_SUCCESS;
+#else 
+  return DRIVER_OP_NO_DRIVER;
+#endif /*HAVE_FREEBSD_CDROM*/
+}
+
 /*!
   Initialization routine. This is the only thing that doesn't
   get called via a function pointer. In fact *we* are the
@@ -580,9 +769,17 @@ cdio_open_am_freebsd (const char *psz_orig_source_name,
   char *psz_source_name;
 
   cdio_funcs_t _funcs = {
-    .eject_media            = _eject_media_freebsd,
-    .free                   = _free_freebsd,
-    .get_arg                = _get_arg_freebsd,
+    .audio_get_volume       = audio_get_volume_freebsd,
+    .audio_pause            = audio_pause_freebsd,
+    .audio_play_msf         = audio_play_msf_freebsd,
+    .audio_play_track_index = audio_play_track_index_freebsd,
+    .audio_read_subchannel  = audio_read_subchannel_freebsd,
+    .audio_resume           = audio_resume_freebsd,
+    .audio_set_volume       = audio_set_volume_freebsd,
+    .audio_stop             = audio_stop_freebsd,
+    .eject_media            = eject_media_freebsd,
+    .free                   = free_freebsd,
+    .get_arg                = get_arg_freebsd,
     .get_blocksize          = get_blocksize_mmc,
     .get_cdtext             = get_cdtext_generic,
     .get_default_device     = cdio_get_default_device_freebsd,
@@ -591,28 +788,29 @@ cdio_open_am_freebsd (const char *psz_orig_source_name,
     .get_discmode           = get_discmode_generic,
     .get_drive_cap          = get_drive_cap_freebsd,
     .get_first_track_num    = get_first_track_num_generic,
-    .get_mcn                = _get_mcn_freebsd,
+    .get_mcn                = get_mcn_freebsd,
     .get_num_tracks         = get_num_tracks_generic,
     .get_track_channels     = get_track_channels_generic,
     .get_track_copy_permit  = get_track_copy_permit_generic,
-    .get_track_format       = _get_track_format_freebsd,
-    .get_track_green        = _get_track_green_freebsd,
-    .get_track_lba          = _get_track_lba_freebsd, 
+    .get_track_format       = get_track_format_freebsd,
+    .get_track_green        = get_track_green_freebsd,
+    .get_track_lba          = get_track_lba_freebsd, 
     .get_track_preemphasis  = get_track_preemphasis_generic,
     .get_track_msf          = NULL,
     .lseek                  = cdio_generic_lseek,
     .read                   = cdio_generic_read,
-    .read_audio_sectors     = _read_audio_sectors_freebsd,
-    .read_mode2_sector      = _read_mode2_sector_freebsd,
-    .read_mode2_sectors     = _read_mode2_sectors_freebsd,
+    .read_audio_sectors     = read_audio_sectors_freebsd,
+    .read_data_sectors      = read_data_sectors_mmc,
+    .read_mode2_sector      = read_mode2_sector_freebsd,
+    .read_mode2_sectors     = read_mode2_sectors_freebsd,
     .read_toc               = read_toc_freebsd,
-    .run_scsi_mmc_cmd       = run_scsi_cmd_freebsd,
-    .set_arg                = _set_arg_freebsd,
-    .set_blocksize         = set_blocksize_mmc,
+    .run_mmc_cmd            = run_mmc_cmd_freebsd,
+    .set_arg                = set_arg_freebsd,
+    .set_blocksize          = set_blocksize_mmc,
     .set_speed              = set_speed_freebsd,
   };
 
-  _data                     = _cdio_malloc (sizeof (_img_private_t));
+  _data                     = calloc(1, sizeof (_img_private_t));
   _data->access_mode        = str_to_access_mode_freebsd(psz_access_mode);
   _data->gen.init           = false;
   _data->gen.fd             = -1;
@@ -624,10 +822,10 @@ cdio_open_am_freebsd (const char *psz_orig_source_name,
     psz_source_name=cdio_get_default_device_freebsd();
     if (NULL == psz_source_name) return NULL;
     _data->device  = psz_source_name;
-    _set_arg_freebsd(_data, "source", psz_source_name);
+    set_arg_freebsd(_data, "source", psz_source_name);
   } else {
     if (cdio_is_device_generic(psz_orig_source_name)) {
-      _set_arg_freebsd(_data, "source", psz_orig_source_name);
+      set_arg_freebsd(_data, "source", psz_orig_source_name);
       _data->device  = strdup(psz_orig_source_name);
     } else {
       /* The below would be okay if all device drivers worked this way. */
