@@ -1,5 +1,5 @@
 /*
-    $Id: win32.c,v 1.14 2005/01/27 11:08:55 rocky Exp $
+    $Id: win32.c,v 1.30 2005/03/09 11:04:34 rocky Exp $
 
     Copyright (C) 2003, 2004, 2005 Rocky Bernstein <rocky@panix.com>
 
@@ -26,12 +26,12 @@
 # include "config.h"
 #endif
 
-static const char _rcsid[] = "$Id: win32.c,v 1.14 2005/01/27 11:08:55 rocky Exp $";
+static const char _rcsid[] = "$Id: win32.c,v 1.30 2005/03/09 11:04:34 rocky Exp $";
 
 #include <cdio/cdio.h>
 #include <cdio/sector.h>
 #include <cdio/util.h>
-#include <cdio/scsi_mmc.h>
+#include <cdio/mmc.h>
 #include "cdio_assert.h"
 #include "cdio_private.h" /* protoype for cdio_is_device_win32 */
 
@@ -84,6 +84,95 @@ static const char _rcsid[] = "$Id: win32.c,v 1.14 2005/01/27 11:08:55 rocky Exp
 #define WIN_NT               ( GetVersion() < 0x80000000 )
 #endif
 
+/*!
+  Pause playing CD through analog output
+  
+  @param p_cdio the CD object to be acted upon.
+*/
+static driver_return_code_t 
+audio_pause_win32 (void *p_user_data)
+{
+  if ( WIN_NT ) {
+    return audio_pause_win32ioctl (p_user_data);
+  } else {
+    return DRIVER_OP_UNSUPPORTED; /* not yet, but soon I hope */
+  }
+}
+
+/*!
+  Playing CD through analog output at the given MSF.
+  
+  @param p_cdio the CD object to be acted upon.
+*/
+static driver_return_code_t 
+audio_play_msf_win32 (void *p_user_data, msf_t *p_start_msf, msf_t *p_end_msf)
+{
+  if ( WIN_NT ) {
+    return audio_play_msf_win32ioctl (p_user_data, p_start_msf, p_end_msf);
+  } else {
+    return DRIVER_OP_UNSUPPORTED; /* not yet, but soon I hope */
+  }
+}
+
+/*!
+  Read Audio Subchannel information
+  
+  @param p_cdio the CD object to be acted upon.
+  
+*/
+static driver_return_code_t
+audio_read_subchannel_win32 (void *p_user_data, 
+			     cdio_subchannel_t *p_subchannel)
+{
+  if ( WIN_NT ) {
+    return audio_read_subchannel_win32ioctl (p_user_data, p_subchannel);
+  } else {
+    return audio_read_subchannel_mmc(p_user_data, p_subchannel);
+  }
+}
+
+  /*!
+    Resume playing an audio CD.
+    
+    @param p_cdio the CD object to be acted upon.
+
+  */
+static driver_return_code_t 
+audio_resume_win32 (void *p_user_data)
+{
+  if ( WIN_NT ) {
+    return audio_resume_win32ioctl (p_user_data);
+  } else {
+    return DRIVER_OP_UNSUPPORTED; /* not yet, but soon I hope */
+  }
+}
+
+/*!
+  Set the volume of an audio CD.
+  
+  @param p_cdio the CD object to be acted upon.
+  
+*/
+static driver_return_code_t
+audio_set_volume_win32 ( void *p_user_data, cdio_audio_volume_t *p_volume)
+{
+  if ( WIN_NT ) {
+    return audio_set_volume_win32ioctl (p_user_data, p_volume);
+  } else {
+    return DRIVER_OP_UNSUPPORTED; /* not yet, but soon I hope */
+  }
+}
+
+static driver_return_code_t
+audio_stop_win32 ( void *p_user_data)
+{
+  if ( WIN_NT ) {
+    return audio_stop_win32ioctl (p_user_data);
+  } else {
+    return DRIVER_OP_UNSUPPORTED; /* not yet, but soon I hope */
+  }
+}
+
 /* General ioctl() CD-ROM command function */
 static bool 
 _cdio_mciSendCommand(int id, UINT msg, DWORD flags, void *arg)
@@ -166,19 +255,19 @@ is_cdrom_win32(const char drive_letter) {
   Return 0 if command completed successfully.
  */
 static int
-run_scsi_cmd_win32( void *p_user_data, unsigned int i_timeout_ms,
-		    unsigned int i_cdb, const scsi_mmc_cdb_t *p_cdb, 
-		    scsi_mmc_direction_t e_direction, 
-		    unsigned int i_buf, /*in/out*/ void *p_buf )
+run_mmc_cmd_win32( void *p_user_data, unsigned int i_timeout_ms,
+		   unsigned int i_cdb, const mmc_cdb_t *p_cdb, 
+		   mmc_direction_t e_direction, 
+		   unsigned int i_buf, /*in/out*/ void *p_buf )
 {
   _img_private_t *p_env = p_user_data;
 
   if (p_env->hASPI) {
-    return run_scsi_cmd_aspi( p_env, i_timeout_ms, i_cdb, p_cdb,  
-			      e_direction, i_buf, p_buf );
+    return run_mmc_cmd_aspi( p_env, i_timeout_ms, i_cdb, p_cdb,  
+			     e_direction, i_buf, p_buf );
   } else {
-    return run_scsi_cmd_win32ioctl( p_env, i_timeout_ms, i_cdb, p_cdb,  
-				    e_direction, i_buf, p_buf );
+    return run_mmc_cmd_win32ioctl( p_env, i_timeout_ms, i_cdb, p_cdb,  
+				   e_direction, i_buf, p_buf );
   }
 }
 
@@ -186,18 +275,20 @@ run_scsi_cmd_win32( void *p_user_data, unsigned int i_timeout_ms,
   Initialize CD device.
  */
 static bool
-_cdio_init_win32 (void *user_data)
+init_win32 (void *p_user_data)
 {
-  _img_private_t *p_env = user_data;
+  _img_private_t *p_env = p_user_data;
+  bool b_ret;
   if (p_env->gen.init) {
     cdio_error ("init called more than once");
     return false;
   }
   
-  p_env->gen.init     = true;
-  p_env->gen.toc_init = false;
+  p_env->gen.init           = true;
+  p_env->gen.toc_init       = false;
   p_env->gen.b_cdtext_init  = false;
   p_env->gen.b_cdtext_error = false;
+  p_env->gen.fd             = open (p_env->gen.source_name, O_RDONLY, 0);
 
   /* Initializations */
   p_env->h_device_handle = NULL;
@@ -207,22 +298,33 @@ _cdio_init_win32 (void *user_data)
   p_env->b_aspi_init     = false;
   p_env->b_ioctl_init    = false;
 
+
   if ( _AM_IOCTL == p_env->access_mode ) {
-    return init_win32ioctl(p_env);
+    b_ret = init_win32ioctl(p_env);
   } else {
-    return init_aspi(p_env);
+    b_ret = init_aspi(p_env);
   }
+
+  /* It looks like get_media_changed_mmc will always
+     return 1 (media changed) on the first call. So 
+     we call it here to clear that flag. We may have
+     to rethink this if there's a problem doing this
+     extra work down the line. */
+  get_media_changed_mmc(p_user_data);
+  return b_ret;
 }
 
 /*!
   Release and free resources associated with cd. 
  */
 static void
-_free_win32 (void *user_data)
+free_win32 (void *p_user_data)
 {
-  _img_private_t *p_env = user_data;
+  _img_private_t *p_env = p_user_data;
 
   if (NULL == p_env) return;
+  if (p_env->gen.fd >= 0) close(p_env->gen.fd);
+
   free (p_env->gen.source_name);
 
   if( p_env->h_device_handle )
@@ -238,8 +340,8 @@ _free_win32 (void *user_data)
    Returns 0 if no error. 
  */
 static int
-_cdio_read_audio_sectors (void *p_user_data, void *p_buf, lsn_t i_lsn, 
-			  unsigned int i_blocks) 
+read_audio_sectors (void *p_user_data, void *p_buf, lsn_t i_lsn, 
+		    unsigned int i_blocks) 
 {
   _img_private_t *p_env = p_user_data;
   if ( p_env->hASPI ) {
@@ -248,7 +350,7 @@ _cdio_read_audio_sectors (void *p_user_data, void *p_buf, lsn_t i_lsn,
 #if 0
     return read_audio_sectors_win32ioctl( p_env, p_buf, i_lsn, i_blocks );
 #else
-    return scsi_mmc_read_sectors( p_env->gen.cdio, p_buf, i_lsn,
+    return mmc_read_sectors( p_env->gen.cdio, p_buf, i_lsn,
                                   CDIO_MMC_READ_TYPE_CDDA, i_blocks);
 #endif
   }
@@ -259,8 +361,8 @@ _cdio_read_audio_sectors (void *p_user_data, void *p_buf, lsn_t i_lsn,
    from lsn. Returns 0 if no error. 
  */
 static int
-_cdio_read_mode1_sector (void *p_user_data, void *p_buf, lsn_t lsn, 
-			 bool b_form2)
+read_mode1_sector (void *p_user_data, void *p_buf, lsn_t lsn, 
+		   bool b_form2)
 {
   _img_private_t *p_env = p_user_data;
 
@@ -291,8 +393,8 @@ _cdio_read_mode1_sector (void *p_user_data, void *p_buf, lsn_t lsn,
    Returns 0 if no error. 
  */
 static int
-_cdio_read_mode1_sectors (void *p_user_data, void *p_buf, lsn_t lsn, 
-			  bool b_form2, unsigned int nblocks)
+read_mode1_sectors (void *p_user_data, void *p_buf, lsn_t lsn, 
+		    bool b_form2, unsigned int nblocks)
 {
   _img_private_t *p_env = p_user_data;
   int i;
@@ -300,13 +402,13 @@ _cdio_read_mode1_sectors (void *p_user_data, void *p_buf, lsn_t lsn,
 
   for (i = 0; i < nblocks; i++) {
     if (b_form2) {
-      if ( (retval = _cdio_read_mode1_sector (p_env, 
+      if ( (retval = read_mode1_sector (p_env, 
 					  ((char *)p_buf) + (M2RAW_SECTOR_SIZE * i),
 					  lsn + i, true)) )
 	return retval;
     } else {
       char buf[M2RAW_SECTOR_SIZE] = { 0, };
-      if ( (retval = _cdio_read_mode1_sector (p_env, buf, lsn + i, false)) )
+      if ( (retval = read_mode1_sector (p_env, buf, lsn + i, false)) )
 	return retval;
       
       memcpy (((char *)p_buf) + (CDIO_CD_FRAMESIZE * i), 
@@ -321,7 +423,7 @@ _cdio_read_mode1_sectors (void *p_user_data, void *p_buf, lsn_t lsn,
    from lsn. Returns 0 if no error. 
  */
 static int
-_cdio_read_mode2_sector (void *p_user_data, void *data, lsn_t lsn, 
+read_mode2_sector (void *p_user_data, void *data, lsn_t lsn, 
 		    bool b_form2)
 {
   char buf[CDIO_CD_FRAMESIZE_RAW] = { 0, };
@@ -361,17 +463,17 @@ _cdio_read_mode2_sector (void *p_user_data, void *data, lsn_t lsn,
    Returns 0 if no error. 
  */
 static int
-_cdio_read_mode2_sectors (void *p_user_data, void *data, lsn_t lsn, 
-			  bool b_form2, unsigned int nblocks)
+read_mode2_sectors (void *p_user_data, void *data, lsn_t lsn, 
+		    bool b_form2, unsigned int i_blocks)
 {
   int i;
   int retval;
   unsigned int blocksize = b_form2 ? M2RAW_SECTOR_SIZE : CDIO_CD_FRAMESIZE;
 
-  for (i = 0; i < nblocks; i++) {
-    if ( (retval = _cdio_read_mode2_sector (p_user_data, 
-					    ((char *)data) + (blocksize * i),
-					    lsn + i, b_form2)) )
+  for (i = 0; i < i_blocks; i++) {
+    if ( (retval = read_mode2_sector (p_user_data, 
+				      ((char *)data) + (blocksize * i),
+				      lsn + i, b_form2)) )
       return retval;
   }
   return 0;
@@ -441,27 +543,22 @@ read_toc_win32 (void *p_user_data)
 }
 
 /*!
-  Eject media. Return 1 if successful, 0 otherwise.
+  Close media tray.
  */
-static int 
-_cdio_eject_media (void *p_user_data) {
+static driver_return_code_t
+open_close_media_win32 (const char *psz_win32_drive, DWORD command_flags) 
+{
 #ifdef _XBOX
   return DRIVER_OP_UNSUPPORTED;
 #else
-  _img_private_t *p_env = p_user_data;
-
-
   MCI_OPEN_PARMS op;
   MCI_STATUS_PARMS st;
   DWORD i_flags;
-  char psz_drive[4];
   int ret;
     
   memset( &op, 0, sizeof(MCI_OPEN_PARMS) );
   op.lpstrDeviceType = (LPCSTR)MCI_DEVTYPE_CD_AUDIO;
-  strcpy( psz_drive, "X:" );
-  psz_drive[0] = p_env->gen.source_name[0];
-  op.lpstrElementName = psz_drive;
+  op.lpstrElementName = psz_win32_drive;
   
   /* Set the flags for the device type */
   i_flags = MCI_OPEN_TYPE | MCI_OPEN_TYPE_ID |
@@ -470,21 +567,45 @@ _cdio_eject_media (void *p_user_data) {
   if( _cdio_mciSendCommand( 0, MCI_OPEN, i_flags, &op ) ) {
     st.dwItem = MCI_STATUS_READY;
     /* Eject disc */
-    ret = _cdio_mciSendCommand( op.wDeviceID, MCI_SET, MCI_SET_DOOR_OPEN, 0 ) != 0;
+    ret = _cdio_mciSendCommand( op.wDeviceID, MCI_SET, command_flags, 0 ) == 0;
     /* Release access to the device */
     _cdio_mciSendCommand( op.wDeviceID, MCI_CLOSE, MCI_WAIT, 0 );
   } else 
-    ret = 0;
+    ret = DRIVER_OP_ERROR;
   
   return ret;
 #endif
 }
 
+/*!
+  Eject media.
+ */
+static driver_return_code_t
+eject_media_win32 (void *p_user_data) 
+{
+  const _img_private_t *p_env = p_user_data;
+  char psz_drive[4];
+  unsigned int i_device = strlen(p_env->gen.source_name);
+    
+  strcpy( psz_drive, "X:" );
+  if (6 == i_device) {
+    psz_drive[0] = p_env->gen.source_name[4];
+  } else if (2 == i_device) {
+    psz_drive[0] = p_env->gen.source_name[0];
+  } else {
+    cdio_info ("Can't pick out drive letter from device %s", 
+	       p_env->gen.source_name);
+    return DRIVER_OP_ERROR;
+  }
+  
+  return open_close_media_win32(psz_drive, MCI_SET_DOOR_OPEN);
+}
+
 /*!
   Return the value associated with the key "arg".
 */
 static const char *
-_get_arg_win32 (void *p_user_data, const char key[])
+get_arg_win32 (void *p_user_data, const char key[])
 {
   _img_private_t *p_env = p_user_data;
 
@@ -511,7 +632,7 @@ _cdio_get_mcn (const void *p_user_data) {
   const _img_private_t *p_env = p_user_data;
 
   if( p_env->hASPI ) {
-    return scsi_mmc_get_mcn( p_env->gen.cdio );
+    return mmc_get_mcn( p_env->gen.cdio );
   } else {
     return get_mcn_win32ioctl(p_env);
   }
@@ -550,9 +671,9 @@ _cdio_get_track_format(void *p_obj, track_t i_track)
   FIXME: there's gotta be a better design for this and get_track_format?
 */
 static bool
-_cdio_get_track_green(void *obj, track_t i_track) 
+_cdio_get_track_green(void *p_obj, track_t i_track) 
 {
-  _img_private_t *p_env = obj;
+  _img_private_t *p_env = p_obj;
   
   switch (_cdio_get_track_format(p_env, i_track)) {
   case TRACK_FORMAT_XA:
@@ -582,11 +703,11 @@ _cdio_get_track_green(void *obj, track_t i_track)
   False is returned if there is no track entry.
 */
 static bool
-_cdio_get_track_msf(void *p_user_data, track_t i_tracks, msf_t *msf)
+_cdio_get_track_msf(void *p_user_data, track_t i_tracks, msf_t *p_msf)
 {
   _img_private_t *p_env = p_user_data;
 
-  if (NULL == msf) return false;
+  if (!p_msf) return false;
 
   if (!p_env->gen.toc_init) 
     if (!read_toc_win32 (p_env)) return false;
@@ -596,7 +717,7 @@ _cdio_get_track_msf(void *p_user_data, track_t i_tracks, msf_t *msf)
   if (i_tracks > p_env->gen.i_tracks+1 || i_tracks == 0) {
     return false;
   } else {
-    cdio_lsn_to_msf(p_env->tocent[i_tracks-1].start_lsn, msf);
+    cdio_lsn_to_msf(p_env->tocent[i_tracks-1].start_lsn, p_msf);
     return true;
   }
 }
@@ -698,12 +819,30 @@ cdio_is_device_win32(const char *source_name)
 #endif
 }
 
+driver_return_code_t
+close_tray_win32 (const char *psz_win32_drive)
+{
+#ifdef HAVE_WIN32_CDROM
+#if 1
+  return open_close_media_win32(psz_win32_drive, MCI_SET_DOOR_CLOSED|MCI_WAIT);
+#else 
+  if ( WIN_NT ) {
+    return close_tray_win32ioctl (psz_win32_drive);
+  } else {
+    return DRIVER_OP_UNSUPPORTED; /* not yet, but soon I hope */
+  }
+#endif
+#else
+  return DRIVER_OP_UNSUPPORTED;
+#endif /* HAVE_WIN32_CDROM*/
+}
+
 /*!
   Initialization routine. This is the only thing that doesn't
   get called via a function pointer. In fact *we* are the
   ones to set that up.
  */
-CdIo *
+CdIo_t *
 cdio_open_win32 (const char *psz_source_name)
 {
 #ifdef HAVE_WIN32_CDROM
@@ -722,12 +861,12 @@ cdio_open_win32 (const char *psz_source_name)
   get called via a function pointer. In fact *we* are the
   ones to set that up.
  */
-CdIo *
+CdIo_t *
 cdio_open_am_win32 (const char *psz_orig_source, const char *psz_access_mode)
 {
 
 #ifdef HAVE_WIN32_CDROM
-  CdIo *ret;
+  CdIo_t *ret;
   _img_private_t *_data;
   char *psz_source;
 
@@ -735,40 +874,51 @@ cdio_open_am_win32 (const char *psz_orig_source, const char *psz_access_mode)
 
   memset( &_funcs, 0, sizeof(_funcs) );
 
-  _funcs.eject_media           = _cdio_eject_media;
-  _funcs.free                  = _free_win32;
-  _funcs.get_arg               = _get_arg_win32;
-  _funcs.get_cdtext            = get_cdtext_generic;
-  _funcs.get_default_device    = cdio_get_default_device_win32;
-  _funcs.get_devices           = cdio_get_devices_win32;
-  _funcs.get_disc_last_lsn     = get_disc_last_lsn_win32;
-  _funcs.get_discmode          = get_discmode_win32;
-  _funcs.get_drive_cap         = get_drive_cap_mmc;
-  _funcs.get_first_track_num   = get_first_track_num_generic;
-  _funcs.get_hwinfo            = NULL;
-  _funcs.get_mcn               = _cdio_get_mcn;
-  _funcs.get_num_tracks        = get_num_tracks_generic;
-  _funcs.get_track_channels    = get_track_channels_generic,
-  _funcs.get_track_copy_permit = get_track_copy_permit_generic,
-  _funcs.get_track_format      = _cdio_get_track_format;
-  _funcs.get_track_green       = _cdio_get_track_green;
-  _funcs.get_track_lba         = NULL; /* This could be done if need be. */
-  _funcs.get_track_msf         = _cdio_get_track_msf;
-  _funcs.get_track_preemphasis = get_track_preemphasis_generic,
-  _funcs.lseek                 = NULL;
-  _funcs.read                  = NULL;
-  _funcs.read_audio_sectors    = _cdio_read_audio_sectors;
-  _funcs.read_mode1_sector     = _cdio_read_mode1_sector;
-  _funcs.read_mode1_sectors    = _cdio_read_mode1_sectors;
-  _funcs.read_mode2_sector     = _cdio_read_mode2_sector;
-  _funcs.read_mode2_sectors    = _cdio_read_mode2_sectors;
-  _funcs.read_toc              = &read_toc_win32;
-  _funcs.run_scsi_mmc_cmd      = &run_scsi_cmd_win32;
-  _funcs.set_arg               = set_arg_win32;
-  _funcs.set_blocksize         = set_blocksize_mmc;
-  _funcs.set_speed             = set_speed_mmc;
+  _funcs.audio_pause            = audio_pause_win32;
+  _funcs.audio_play_msf         = audio_play_msf_win32;
+#if 0
+  _funcs.audio_play_track_index = audio_play_track_index_win32;
+#endif
+  _funcs.audio_read_subchannel  = audio_read_subchannel_win32;
+  _funcs.audio_resume           = audio_resume_win32;
+  _funcs.audio_set_volume       = audio_set_volume_win32;
+  _funcs.audio_stop             = audio_stop_win32;
+  _funcs.eject_media            = eject_media_win32;
+  _funcs.free                   = free_win32;
+  _funcs.get_arg                = get_arg_win32;
+  _funcs.get_cdtext             = get_cdtext_generic;
+  _funcs.get_default_device     = cdio_get_default_device_win32;
+  _funcs.get_devices            = cdio_get_devices_win32;
+  _funcs.get_disc_last_lsn      = get_disc_last_lsn_win32;
+  _funcs.get_discmode           = get_discmode_win32;
+  _funcs.get_drive_cap          = get_drive_cap_mmc;
+  _funcs.get_first_track_num    = get_first_track_num_generic;
+  _funcs.get_hwinfo             = NULL;
+  _funcs.get_media_changed      = get_media_changed_mmc;
+  _funcs.get_mcn                = _cdio_get_mcn;
+  _funcs.get_num_tracks         = get_num_tracks_generic;
+  _funcs.get_track_channels     = get_track_channels_generic,
+  _funcs.get_track_copy_permit  = get_track_copy_permit_generic,
+  _funcs.get_track_format       = _cdio_get_track_format;
+  _funcs.get_track_green        = _cdio_get_track_green;
+  _funcs.get_track_lba          = NULL; /* This could be done if need be. */
+  _funcs.get_track_msf          = _cdio_get_track_msf;
+  _funcs.get_track_preemphasis  = get_track_preemphasis_generic,
+  _funcs.lseek                  = NULL;
+  _funcs.read                   = NULL;
+  _funcs.read_audio_sectors     = read_audio_sectors;
+  _funcs.read_data_sectors      = read_data_sectors_generic;
+  _funcs.read_mode1_sector      = read_mode1_sector;
+  _funcs.read_mode1_sectors     = read_mode1_sectors;
+  _funcs.read_mode2_sector      = read_mode2_sector;
+  _funcs.read_mode2_sectors     = read_mode2_sectors;
+  _funcs.read_toc               = read_toc_win32;
+  _funcs.run_mmc_cmd            = run_mmc_cmd_win32;
+  _funcs.set_arg                = set_arg_win32;
+  _funcs.set_blocksize          = set_blocksize_mmc;
+  _funcs.set_speed              = set_speed_mmc;
 
-  _data                 = _cdio_malloc (sizeof (_img_private_t));
+  _data                 = calloc(1, sizeof (_img_private_t));
   _data->access_mode    = str_to_access_mode_win32(psz_access_mode);
   _data->gen.init       = false;
   _data->gen.fd         = -1;
@@ -794,10 +944,10 @@ cdio_open_am_win32 (const char *psz_orig_source, const char *psz_access_mode)
   ret = cdio_new ((void *)_data, &_funcs);
   if (ret == NULL) return NULL;
 
-  if (_cdio_init_win32(_data))
+  if (init_win32(_data))
     return ret;
   else {
-    _free_win32 (_data);
+    free_win32 (_data);
     return NULL;
   }
   
