@@ -1,5 +1,5 @@
 /*
-    $Id: device.c,v 1.8 2005/01/26 01:03:16 rocky Exp $
+    $Id: device.c,v 1.27 2005/04/11 01:03:46 rocky Exp $
 
     Copyright (C) 2005 Rocky Bernstein <rocky@panix.com>
 
@@ -37,6 +37,14 @@
 #include <string.h>
 #endif
 
+/* This probably will get moved to driver code, i.e _cdio_linux.c */
+#ifdef HAVE_LINUX_MAJOR_H
+#include <linux/major.h>
+#endif
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
+
 /* The below array gives of the drivers that are currently available for 
    on a particular host. */
 
@@ -87,6 +95,7 @@ CdIo_driver_t CdIo_all_drivers[CDIO_MAX_DRIVER+1] = {
    NULL,
    NULL,
    NULL,
+   NULL,
    NULL
   },
 
@@ -99,7 +108,8 @@ CdIo_driver_t CdIo_all_drivers[CDIO_MAX_DRIVER+1] = {
    &cdio_open_am_aix,
    &cdio_get_default_device_aix,
    &cdio_is_device_generic,
-   &cdio_get_devices_aix
+   &cdio_get_devices_aix,
+   NULL
   },
 
   {DRIVER_BSDI, 
@@ -111,7 +121,8 @@ CdIo_driver_t CdIo_all_drivers[CDIO_MAX_DRIVER+1] = {
    &cdio_open_am_bsdi,
    &cdio_get_default_device_bsdi,
    &cdio_is_device_generic,
-   &cdio_get_devices_bsdi
+   &cdio_get_devices_bsdi,
+   &close_tray_bsdi
   },
 
   {DRIVER_FREEBSD, 
@@ -123,7 +134,8 @@ CdIo_driver_t CdIo_all_drivers[CDIO_MAX_DRIVER+1] = {
    &cdio_open_am_freebsd,
    &cdio_get_default_device_freebsd,
    &cdio_is_device_generic,
-   NULL
+   &cdio_get_devices_freebsd,
+   &close_tray_freebsd
   },
 
   {DRIVER_LINUX, 
@@ -135,7 +147,8 @@ CdIo_driver_t CdIo_all_drivers[CDIO_MAX_DRIVER+1] = {
    &cdio_open_am_linux,
    &cdio_get_default_device_linux,
    &cdio_is_device_generic,
-   &cdio_get_devices_linux
+   &cdio_get_devices_linux,
+   &close_tray_linux
   },
 
   {DRIVER_SOLARIS, 
@@ -147,7 +160,8 @@ CdIo_driver_t CdIo_all_drivers[CDIO_MAX_DRIVER+1] = {
    &cdio_open_am_solaris,
    &cdio_get_default_device_solaris,
    &cdio_is_device_generic,
-   &cdio_get_devices_solaris
+   &cdio_get_devices_solaris,
+   &close_tray_solaris
   },
 
   {DRIVER_OSX, 
@@ -159,7 +173,8 @@ CdIo_driver_t CdIo_all_drivers[CDIO_MAX_DRIVER+1] = {
    &cdio_open_am_osx,
    &cdio_get_default_device_osx,
    &cdio_is_device_generic,
-   &cdio_get_devices_osx
+   &cdio_get_devices_osx,
+   &close_tray_osx
   },
 
   {DRIVER_WIN32, 
@@ -171,7 +186,8 @@ CdIo_driver_t CdIo_all_drivers[CDIO_MAX_DRIVER+1] = {
    &cdio_open_am_win32,
    &cdio_get_default_device_win32,
    &cdio_is_device_win32,
-   &cdio_get_devices_win32
+   &cdio_get_devices_win32,
+   &close_tray_win32
   },
 
   {DRIVER_CDRDAO,
@@ -183,7 +199,8 @@ CdIo_driver_t CdIo_all_drivers[CDIO_MAX_DRIVER+1] = {
    &cdio_open_am_cdrdao,
    &cdio_get_default_device_cdrdao,
    NULL,
-   &cdio_get_devices_cdrdao
+   &cdio_get_devices_cdrdao,
+   NULL
   },
 
   {DRIVER_BINCUE,
@@ -195,7 +212,8 @@ CdIo_driver_t CdIo_all_drivers[CDIO_MAX_DRIVER+1] = {
    &cdio_open_am_bincue,
    &cdio_get_default_device_bincue,
    NULL,
-   &cdio_get_devices_bincue
+   &cdio_get_devices_bincue,
+   NULL
   },
 
   {DRIVER_NRG,
@@ -207,7 +225,8 @@ CdIo_driver_t CdIo_all_drivers[CDIO_MAX_DRIVER+1] = {
    &cdio_open_am_nrg,
    &cdio_get_default_device_nrg,
    NULL,
-   &cdio_get_devices_nrg
+   &cdio_get_devices_nrg,
+   NULL
   }
 
 };
@@ -254,7 +273,7 @@ cdio_init(void)
     return false;
   }
 
-  for (driver_id=DRIVER_UNKNOWN; driver_id<=CDIO_MAX_DRIVER; driver_id++) {
+  for (driver_id=DRIVER_UNKNOWN+1; driver_id<=CDIO_MAX_DRIVER; driver_id++) {
     all_dp = &CdIo_all_drivers[driver_id];
     if ((*CdIo_all_drivers[driver_id].have_driver)()) {
       *dp++ = *all_dp;
@@ -274,23 +293,60 @@ cdio_destroy (CdIo_t *p_cdio)
   CdIo_last_driver = CDIO_DRIVER_UNINIT;
   if (p_cdio == NULL) return;
 
-  if (p_cdio->op.free != NULL) 
+  if (p_cdio->op.free != NULL && p_cdio->env) 
     p_cdio->op.free (p_cdio->env);
   p_cdio->env = NULL;
   free (p_cdio);
 }
 
-  /*!
-    Eject media in CD drive if there is a routine to do so. 
+/*!
+  Close media tray in CD drive if there is a routine to do so. 
+  
+  @param psz_drive the name of CD-ROM to be closed.
+  @param p_driver_id is the driver to be used or that got used if
+  it was DRIVER_UNKNOWN or DRIVER_DEVICE; If this is NULL, we won't
+  report back the driver used.
+*/
+driver_return_code_t 
+cdio_close_tray (const char *psz_drive, /*in/out*/ driver_id_t
+                 *p_driver_id)
+{
+  driver_id_t temp_driver_id = DRIVER_DEVICE;
 
-    @param p_cdio the CD object to be acted upon.
-    If the CD is ejected *p_cdio is freed and p_cdio set to NULL.
-  */
+  if (!p_driver_id) p_driver_id = &temp_driver_id;
+  
+  if (DRIVER_UNKNOWN == *p_driver_id || DRIVER_DEVICE == *p_driver_id) {
+    *p_driver_id = CDIO_MIN_DEVICE_DRIVER;
+    
+    /* Scan for driver */
+    for ( ; *p_driver_id<=CDIO_MAX_DRIVER; (*p_driver_id)++) {
+      if ( (*CdIo_all_drivers[*p_driver_id].have_driver)() &&
+           *CdIo_all_drivers[*p_driver_id].close_tray ) {
+        return (*CdIo_all_drivers[*p_driver_id].close_tray)(psz_drive);
+      }
+    }
+    return DRIVER_OP_UNSUPPORTED;
+  }
+  
+  /* The driver id was specified. Use that. */
+  if ( (*CdIo_all_drivers[*p_driver_id].have_driver)() &&
+       *CdIo_all_drivers[*p_driver_id].close_tray ) {
+    return (*CdIo_all_drivers[*p_driver_id].close_tray)(psz_drive);
+  }
+  return DRIVER_OP_UNSUPPORTED;
+}
+
+/*!
+  Eject media in CD drive if there is a routine to do so. 
+  
+  @param p_cdio the CD object to be acted upon.
+  If the CD is ejected *p_cdio is freed and p_cdio set to NULL.
+*/
 driver_return_code_t
 cdio_eject_media (CdIo_t **pp_cdio)
 {
   
-  if ((pp_cdio == NULL) || (*pp_cdio == NULL)) return 1;
+  if ((pp_cdio == NULL) || (*pp_cdio == NULL)) return DRIVER_OP_UNINIT;
 
   if ((*pp_cdio)->op.eject_media) {
     int ret = (*pp_cdio)->op.eject_media ((*pp_cdio)->env);
@@ -310,17 +366,22 @@ cdio_eject_media (CdIo_t **pp_cdio)
   Free device list returned by cdio_get_devices or
   cdio_get_devices_with_cap.
 */
-void cdio_free_device_list (char * device_list[]) 
+void 
+cdio_free_device_list (char * ppsz_device_list[]) 
 {
-  if (NULL == device_list) return;
-  for ( ; *device_list != NULL ; device_list++ )
-    free(*device_list);
+  char **ppsz_device_list_save=ppsz_device_list;
+  if (!ppsz_device_list) return;
+  for ( ; NULL != *ppsz_device_list ; ppsz_device_list++ ) {
+    free(*ppsz_device_list);
+    *ppsz_device_list = NULL;
+  }
+  free(ppsz_device_list_save);
 }
 
 
 /*!
   Return a string containing the default CD device if none is specified.
-  if CdIo is NULL (we haven't initialized a specific device driver), 
+  if p_cdio is NULL (we haven't initialized a specific device driver), 
   then find a suitable one and return the default device for that.
 
   NULL is returned if we couldn't get a default device.
@@ -331,7 +392,7 @@ cdio_get_default_device (const CdIo_t *p_cdio)
   if (p_cdio == NULL) {
     driver_id_t driver_id;
     /* Scan for driver */
-    for (driver_id=DRIVER_UNKNOWN; driver_id<=CDIO_MAX_DRIVER; driver_id++) {
+    for (driver_id=DRIVER_UNKNOWN+1; driver_id<=CDIO_MAX_DRIVER; driver_id++) {
       if ( (*CdIo_all_drivers[driver_id].have_driver)() &&
            *CdIo_all_drivers[driver_id].get_default_device ) {
         return (*CdIo_all_drivers[driver_id].get_default_device)();
@@ -347,6 +408,40 @@ cdio_get_default_device (const CdIo_t *p_cdio)
   }
 }
 
+/*!
+  Return a string containing the default CD device if none is specified.
+  if p_driver_id is DRIVER_UNKNOWN or DRIVER_DEVICE
+  then find a suitable one set the default device for that.
+
+  NULL is returned if we couldn't get a default device.
+ */
+char *
+cdio_get_default_device_driver (/*in/out*/ driver_id_t *p_driver_id)
+{
+  if (DRIVER_UNKNOWN == *p_driver_id || DRIVER_DEVICE == *p_driver_id) {
+    if (DRIVER_UNKNOWN == *p_driver_id) 
+      (*p_driver_id)++;
+    else 
+      *p_driver_id = CDIO_MIN_DEVICE_DRIVER;
+    
+    /* Scan for driver */
+    for ( ; *p_driver_id<=CDIO_MAX_DRIVER; (*p_driver_id)++) {
+      if ( (*CdIo_all_drivers[*p_driver_id].have_driver)() &&
+           *CdIo_all_drivers[*p_driver_id].get_default_device ) {
+        return (*CdIo_all_drivers[*p_driver_id].get_default_device)();
+      }
+    }
+    return NULL;
+  }
+  
+  /* The driver id was specified. Use that. */
+  if ( (*CdIo_all_drivers[*p_driver_id].have_driver)() &&
+       *CdIo_all_drivers[*p_driver_id].get_default_device ) {
+    return (*CdIo_all_drivers[*p_driver_id].get_default_device)();
+  }
+  return NULL;
+}
+
 /*!Return an array of device names. If you want a specific
   devices, dor a driver give that device, if you want hardware
   devices, give DRIVER_DEVICE and if you want all possible devices,
@@ -370,12 +465,12 @@ cdio_get_devices_ret (/*in/out*/ driver_id_t *p_driver_id)
   switch (*p_driver_id) {
     /* FIXME: spit out unknown to give image drivers as well.  */
   case DRIVER_DEVICE:
-    p_cdio = scan_for_driver(DRIVER_UNKNOWN, CDIO_MAX_DEVICE_DRIVER, 
+    p_cdio = scan_for_driver(DRIVER_UNKNOWN+1, CDIO_MAX_DEVICE_DRIVER, 
                              NULL, NULL);
     *p_driver_id = cdio_get_driver_id(p_cdio);
     break;
   case DRIVER_UNKNOWN:
-    p_cdio = scan_for_driver(DRIVER_UNKNOWN, CDIO_MAX_DRIVER, NULL, NULL);
+    p_cdio = scan_for_driver(DRIVER_UNKNOWN+1, CDIO_MAX_DRIVER, NULL, NULL);
     *p_driver_id = cdio_get_driver_id(p_cdio);
     break;
   default:
@@ -409,7 +504,7 @@ cdio_get_devices_ret (/*in/out*/ driver_id_t *p_driver_id)
   the value is NULL. This also means nothing was found.
 */
 char **
-cdio_get_devices_with_cap (/*out*/ char* search_devices[], 
+cdio_get_devices_with_cap (/*in*/ char* search_devices[], 
                            cdio_fs_anal_t capabilities, bool any)
 {
   driver_id_t p_driver_id;
@@ -418,18 +513,23 @@ cdio_get_devices_with_cap (/*out*/ char* search_devices[],
 }
 
 char **
-cdio_get_devices_with_cap_ret (/*out*/ char* search_devices[], 
+cdio_get_devices_with_cap_ret (/*in*/ char* search_devices[], 
                                cdio_fs_anal_t capabilities, bool any,
                                /*out*/ driver_id_t *p_driver_id)
 {
   char **ppsz_drives=search_devices;
   char **ppsz_drives_ret=NULL;
   unsigned int i_drives=0;
+  bool b_free_ppsz_drives = false;
 
   *p_driver_id = DRIVER_DEVICE;
 
-  if (NULL == ppsz_drives) ppsz_drives=cdio_get_devices_ret(p_driver_id);
-  if (NULL == ppsz_drives) return NULL;
+  if (!ppsz_drives) {
+    ppsz_drives=cdio_get_devices_ret(p_driver_id);
+    b_free_ppsz_drives = true;
+  }
+  
+  if (!ppsz_drives) return NULL;
 
   if (capabilities == CDIO_FS_MATCH_ALL) {
     /* Duplicate drives into drives_ret. */
@@ -471,8 +571,9 @@ cdio_get_devices_with_cap_ret (/*out*/ char* search_devices[],
     }
   }
   cdio_add_device_list(&ppsz_drives_ret, NULL, &i_drives);
-  cdio_free_device_list(ppsz_drives);
-  free(ppsz_drives);
+  if (b_free_ppsz_drives) {
+    cdio_free_device_list(ppsz_drives);
+  }
   return ppsz_drives_ret;
 }
 
@@ -564,10 +665,110 @@ cdio_get_hwinfo (const CdIo_t *p_cdio, cdio_hwinfo_t *hw_info)
   } else {
     /* Perhaps driver forgot to initialize.  We are no worse off Using
        scsi_mmc than returning false here. */
-    return scsi_mmc_get_hwinfo(p_cdio, hw_info);
+    return mmc_get_hwinfo(p_cdio, hw_info);
   }
 }
 
+/*!
+  Return the session number of the last on the CD. 
+  
+  @param p_cdio the CD object to be acted upon.
+  @param i_last_session pointer to the session number to be returned.
+*/
+driver_return_code_t cdio_get_last_session (CdIo_t *p_cdio,
+                                            /*out*/ lsn_t *i_last_session)
+{
+  if (!p_cdio) return DRIVER_OP_UNINIT;
+  if (p_cdio->op.get_last_session)
+    return p_cdio->op.get_last_session(p_cdio->env, i_last_session);
+  return DRIVER_OP_UNSUPPORTED;
+}
+
+/*! 
+  Find out if media has changed since the last call.
+  @param p_cdio the CD object to be acted upon.
+  @return 1 if media has changed since last call, 0 if not. Error
+  return codes are the same as driver_return_code_t
+*/
+int
+cdio_get_media_changed(CdIo_t *p_cdio)
+{
+  if (!p_cdio) return DRIVER_OP_UNINIT;
+  if (p_cdio->op.get_media_changed)
+    return p_cdio->op.get_media_changed(p_cdio->env);
+  return DRIVER_OP_UNSUPPORTED;
+}
+
+bool_3way_t
+cdio_have_atapi(CdIo_t *p_cdio)
+{
+  bool_3way_t i_status;
+
+  if (!p_cdio) return nope;
+  i_status = mmc_have_interface(p_cdio, CDIO_MMC_FEATURE_INTERFACE_ATAPI);
+  if (dunno != i_status) return i_status;
+
+  {
+    /* cdparanoia seems to think that if we have a mode sense command
+       we have an atapi drive or is atapi compatible.
+     */
+    uint8_t buf[22];
+    if (DRIVER_OP_SUCCESS == mmc_mode_sense(p_cdio, buf, sizeof(buf),
+                                            CDIO_MMC_CAPABILITIES_PAGE) ) {
+      uint8_t *b = buf;
+      b+=b[3]+4;
+      if( CDIO_MMC_CAPABILITIES_PAGE == (b[0]&0x3F) ) {
+        /* MMC style drive! */
+        return yep;
+      }
+    }
+  }
+  
+  /* Put these in the various drivers? If we get more, yes!
+   */
+#ifdef HAVE_LINUX_MAJOR_H
+  {
+    /* This too is from cdparanoia. */
+    struct stat st;
+    generic_img_private_t *p_env  = p_cdio->env;
+    if ( 0 == lstat(p_env->source_name, &st) ) {
+      if (S_ISCHR(st.st_mode) || S_ISBLK(st.st_mode)) {
+        int drive_type=(int)(st.st_rdev>>8);
+        switch (drive_type) {
+        case IDE0_MAJOR:
+        case IDE1_MAJOR:
+        case IDE2_MAJOR:
+        case IDE3_MAJOR:
+          /* Yay, ATAPI... */
+          return yep;
+          break;
+        case CDU31A_CDROM_MAJOR:
+        case CDU535_CDROM_MAJOR:
+        case MATSUSHITA_CDROM_MAJOR:
+        case MATSUSHITA_CDROM2_MAJOR:
+        case MATSUSHITA_CDROM3_MAJOR:
+        case MATSUSHITA_CDROM4_MAJOR:
+        case SANYO_CDROM_MAJOR:
+        case MITSUMI_CDROM_MAJOR:
+        case MITSUMI_X_CDROM_MAJOR:
+        case OPTICS_CDROM_MAJOR:
+        case AZTECH_CDROM_MAJOR:
+        case GOLDSTAR_CDROM_MAJOR:
+        case CM206_CDROM_MAJOR:
+        case SCSI_CDROM_MAJOR:   
+        case SCSI_GENERIC_MAJOR: 
+          return nope;
+	break;
+        default:
+          return dunno;
+        }
+      }
+    }
+  }
+#endif /*HAVE_LINUX_MAJOR_H*/
+  return dunno;
+}
+
 bool
 cdio_have_driver(driver_id_t driver_id)
 {
@@ -581,6 +782,7 @@ cdio_is_device(const char *psz_source, driver_id_t driver_id)
   return (*CdIo_all_drivers[driver_id].is_device)(psz_source);
 }
 
+
 /*! Sets up to read from place specified by source_name and
   driver_id. This should be called before using any other routine,
   except cdio_init. This will call cdio_init, if that hasn't been
@@ -617,10 +819,10 @@ cdio_open_am (const char *psz_orig_source, driver_id_t driver_id,
   switch (driver_id) {
   case DRIVER_UNKNOWN: 
     {
-      CdIo_t *cdio=scan_for_driver(CDIO_MIN_DRIVER, CDIO_MAX_DRIVER, 
-                                 psz_source, psz_access_mode);
+      CdIo_t *p_cdio=scan_for_driver(CDIO_MIN_DRIVER, CDIO_MAX_DRIVER, 
+                                     psz_source, psz_access_mode);
       free(psz_source);
-      return cdio;
+      return p_cdio;
     }
   case DRIVER_DEVICE: 
     {  
@@ -707,7 +909,7 @@ cdio_set_blocksize ( const CdIo_t *p_cdio, int i_blocksize )
 /*!
   Set the drive speed. 
   
-  @see cdio_get_speed
+  @see cdio_set_speed
 */
 driver_return_code_t
 cdio_set_speed (const CdIo_t *p_cdio, int i_speed) 
