@@ -1,5 +1,5 @@
 /*
-    $Id: cdrdao.c,v 1.11 2005/01/24 00:06:31 rocky Exp $
+    $Id: cdrdao.c,v 1.17 2005/04/09 22:56:07 rocky Exp $
 
     Copyright (C) 2004, 2005 Rocky Bernstein <rocky@panix.com>
     toc reading routine adapted from cuetools
@@ -25,7 +25,7 @@
    (*.cue).
 */
 
-static const char _rcsid[] = "$Id: cdrdao.c,v 1.11 2005/01/24 00:06:31 rocky Exp $";
+static const char _rcsid[] = "$Id: cdrdao.c,v 1.17 2005/04/09 22:56:07 rocky Exp $";
 
 #include "image.h"
 #include "cdio_assert.h"
@@ -73,12 +73,12 @@ static bool parse_tocfile (_img_private_t *cd, const char *p_toc_name);
 static bool
 check_track_is_blocksize_multiple(const char *psz_fname, 
 				  track_t i_track, long i_size, 
-				  long i_blocksize)
+				  uint16_t i_blocksize)
 {
   if (i_size % i_blocksize) {
     cdio_info ("image %s track %d size (%ld) not a multiple"
 	       " of the blocksize (%ld)", psz_fname, i_track, i_size, 
-	       i_blocksize);
+	       (long int) i_blocksize);
     if (i_size % M2RAW_SECTOR_SIZE == 0)
       cdio_info ("this may be a 2336-type disc image");
     else if (i_size % CDIO_CD_FRAMESIZE_RAW == 0)
@@ -236,7 +236,7 @@ get_disc_last_lsn_cdrdao (void *p_user_data)
 {
   _img_private_t *p_env = p_user_data;
   track_t i_leadout = p_env->gen.i_tracks;
-  long i_blocksize  = p_env->tocent[i_leadout-1].blocksize;
+  uint16_t i_blocksize  = p_env->tocent[i_leadout-1].blocksize;
   long i_size;
 
   if (p_env->tocent[i_leadout-1].sec_count) {
@@ -787,7 +787,7 @@ parse_tocfile (_img_private_t *cd, const char *psz_cue_name)
 	    /* No start-msf. */
 	    if (cd) {
 	      if (i) {
-		long i_blocksize = cd->tocent[i-1].blocksize;
+		uint16_t i_blocksize = cd->tocent[i-1].blocksize;
 		long i_size      = 
 		  cdio_stream_stat(cd->tocent[i-1].data_source);
 
@@ -1142,7 +1142,7 @@ cdio_get_default_device_cdrdao(void)
 }
 
 static bool
-get_hwinfo_cdrdao ( const CdIo *p_cdio, /*out*/ cdio_hwinfo_t *hw_info)
+get_hwinfo_cdrdao ( const CdIo_t *p_cdio, /*out*/ cdio_hwinfo_t *hw_info)
 {
   strcpy(hw_info->psz_vendor, "libcdio");
   strcpy(hw_info->psz_model, "cdrdao");
@@ -1238,7 +1238,7 @@ cdio_is_tocfile(const char *psz_cue_name)
   get called via a function pointer. In fact *we* are the
   ones to set that up.
  */
-CdIo *
+CdIo_t *
 cdio_open_am_cdrdao (const char *psz_source_name, const char *psz_access_mode)
 {
   if (psz_access_mode != NULL && strcmp(psz_access_mode, "image"))
@@ -1252,11 +1252,11 @@ cdio_open_am_cdrdao (const char *psz_source_name, const char *psz_access_mode)
   get called via a function pointer. In fact *we* are the
   ones to set that up.
  */
-CdIo *
+CdIo_t *
 cdio_open_cdrdao (const char *psz_cue_name)
 {
-  CdIo *ret;
-  _img_private_t *_data;
+  CdIo_t *ret;
+  _img_private_t *p_data;
 
   cdio_funcs_t _funcs;
   
@@ -1273,6 +1273,7 @@ cdio_open_cdrdao (const char *psz_cue_name)
   _funcs.get_drive_cap         = _get_drive_cap_image;
   _funcs.get_first_track_num   = _get_first_track_num_image;
   _funcs.get_hwinfo            = get_hwinfo_cdrdao;
+  _funcs.get_media_changed     = get_media_changed_image;
   _funcs.get_mcn               = _get_mcn_image;
   _funcs.get_num_tracks        = _get_num_tracks_image;
   _funcs.get_track_channels    = get_track_channels_image,
@@ -1285,26 +1286,28 @@ cdio_open_cdrdao (const char *psz_cue_name)
   _funcs.lseek                 = _lseek_cdrdao;
   _funcs.read                  = _read_cdrdao;
   _funcs.read_audio_sectors    = _read_audio_sectors_cdrdao;
+  _funcs.read_data_sectors     = read_data_sectors_image;
   _funcs.read_mode1_sector     = _read_mode1_sector_cdrdao;
   _funcs.read_mode1_sectors    = _read_mode1_sectors_cdrdao;
   _funcs.read_mode2_sector     = _read_mode2_sector_cdrdao;
   _funcs.read_mode2_sectors    = _read_mode2_sectors_cdrdao;
+  _funcs.run_mmc_cmd           = NULL;
   _funcs.set_arg               = _set_arg_image;
   _funcs.set_speed             = cdio_generic_unimplemented_set_speed;
   _funcs.set_blocksize         = cdio_generic_unimplemented_set_blocksize;
 
   if (NULL == psz_cue_name) return NULL;
   
-  _data                  = _cdio_malloc (sizeof (_img_private_t));
-  _data->gen.init        = false;
-  _data->psz_cue_name    = NULL;
-  _data->gen.data_source = NULL;
-  _data->gen.source_name = NULL;
+  p_data                  = calloc(1, sizeof (_img_private_t));
+  p_data->gen.init        = false;
+  p_data->psz_cue_name    = NULL;
+  p_data->gen.data_source = NULL;
+  p_data->gen.source_name = NULL;
 
-  ret = cdio_new ((void *)_data, &_funcs);
+  ret = cdio_new ((void *)p_data, &_funcs);
 
   if (ret == NULL) {
-    free(_data);
+    free(p_data);
     return NULL;
   }
 
@@ -1312,17 +1315,18 @@ cdio_open_cdrdao (const char *psz_cue_name)
   if (!cdio_is_tocfile(psz_cue_name)) {
     cdio_debug ("source name %s is not recognized as a TOC file", 
 		psz_cue_name);
+    free(p_data);
     return NULL;
   }
   
-  _set_arg_image (_data, "cue", psz_cue_name);
-  _set_arg_image (_data, "source", psz_cue_name);
-  _set_arg_image (_data, "access-mode", "cdrdao");
+  _set_arg_image (p_data, "cue", psz_cue_name);
+  _set_arg_image (p_data, "source", psz_cue_name);
+  _set_arg_image (p_data, "access-mode", "cdrdao");
 
-  if (_init_cdrdao(_data)) {
+  if (_init_cdrdao(p_data)) {
     return ret;
   } else {
-    _free_image(_data);
+    _free_image(p_data);
     free(ret);
     return NULL;
   }
