@@ -1,5 +1,5 @@
 /*
-  $Id: cd-drive.c,v 1.17 2005/01/22 22:21:36 rocky Exp $
+  $Id: cd-drive.c,v 1.19 2005/04/11 01:37:38 rocky Exp $
 
   Copyright (C) 2004, 2005 Rocky Bernstein <rocky@panix.com>
   
@@ -131,8 +131,6 @@ parse_options (int argc, const char *argv[])
 
 /* CDIO logging routines */
 
-static cdio_log_handler_t gl_default_cdio_log_handler = NULL;
-
 static void 
 _log_handler (cdio_log_level_t level, const char message[])
 {
@@ -249,7 +247,6 @@ main(int argc, const char *argv[])
       }
     
     cdio_free_device_list(ppsz_cdrives);
-    free(ppsz_cdrives);
     ppsz_cdrives = NULL;
   } else {
     /* Print CD-drive info for given source */
