@@ -1,19 +1,23 @@
 #!/bin/sh
-#$Id: check_cue.sh.in,v 1.25 2005/01/29 14:50:34 rocky Exp $
+#$Id: check_cue.sh.in,v 1.28 2005/04/11 02:36:36 rocky Exp $
 # Tests to see that BIN/CUE and cdrdao TOC file iamge reading is correct
 # (via cd-info).
 
-if test -n "-L/usr/local/lib -lvcdinfo -liso9660 -lcdio -lm  " ; then
+if test "-L/usr/local/lib -lvcdinfo -liso9660 -lcdio -lm  X" != "X" ; then
   vcd_opt='--no-vcd'
 fi
 
-if test -z "$srcdir" ; then
+if test "X$srcdir" = "X" ; then
   srcdir=`pwd`
 fi
 
+if test "X$top_srcdir" = "X" ; then
+  top_srcdir=`pwd`/..
+fi
+
 . ${srcdir}/check_common_fn
 
-if test ! -x ../src/cd-info ; then
+if test ! -x $top_srcdir/src/cd-info ; then
   exit 77
 fi
 
@@ -43,7 +47,13 @@ if test -f  ${srcdir}/${fname}.bin ; then
   opts="-q --no-device-info --no-disc-mode --cue-file ${srcdir}/${fname}.cue --iso9660"
   test_cdinfo "$opts" ${fname}.dump ${srcdir}/${fname}.right
   RC=$?
-  check_result $RC "cd-info CUE test $testnum" "${CD_INFO} $opts"
+  check_result $RC "cd-info Rock-Ridge CUE test $testnum" "${CD_INFO} $opts"
+
+  opts="-q --no-device-info --no-disc-mode --no-rock-ridge --cue-file ${srcdir}/${fname}.cue --iso9660"
+  test_cdinfo "$opts" ${fname}.dump ${srcdir}/${fname}-no-rr.right
+  RC=$?
+  check_result $RC "cd-info no Rock-Ridge CUE test $testnum" "${CD_INFO} $opts"
+
 else 
   echo "Don't see CUE file ${srcdir}/${fname}.bin. Test $testnum skipped."
 fi
