@@ -1,5 +1,5 @@
 #!/bin/sh
-#$Id: check_iso.sh.in,v 1.5 2005/01/29 14:50:34 rocky Exp $
+#$Id: check_iso.sh.in,v 1.9 2005/03/05 23:51:40 rocky Exp $
 
 if test -z $srcdir ; then
   srcdir=`pwd`
@@ -14,12 +14,23 @@ fi
 BASE=`basename $0 .sh`
 fname=copying
 
-if test -n "" ; then
-  opts="--quiet ${srcdir}/${fname}.iso --iso9660 "
-  test_iso_info  "$opts" ${fname}.dump ${srcdir}/${fname}.right
-  RC=$?
-  check_result $RC 'iso-info basic test' "$ISO_INFO $opts"
+opts="--quiet ${srcdir}/${fname}.iso --iso9660 "
+test_iso_info  "$opts" ${fname}.dump ${srcdir}/${fname}.right
+RC=$?
+check_result $RC 'iso-info basic test' "$ISO_INFO $opts"
 
+opts="--image ${srcdir}/${fname}.iso --extract $fname "
+test_iso_read  "$opts" ${fname} ${srcdir}/../COPYING
+RC=$?
+check_result $RC 'iso-read basic test' "$ISO_READ $opts"
+
+fname=copying-rr
+opts="--quiet ${srcdir}/${fname}.iso --iso9660 "
+test_iso_info  "$opts" ${fname}.dump ${srcdir}/${fname}.right
+RC=$?
+check_result $RC 'iso-info Rock Ridge test' "$ISO_INFO $opts"
+
+if test -n "1" ; then
   BASE=`basename $0 .sh`
   fname=joliet
   opts="--quiet ${srcdir}/${fname}.iso --iso9660 "
@@ -33,11 +44,6 @@ if test -n "" ; then
   check_result $RC 'iso-info --no-joliet test' "$cmdline"
 fi
 
-opts="--image ${srcdir}/${fname}.iso --extract $fname "
-test_iso_read  "$opts" ${fname} ${srcdir}/../COPYING
-RC=$?
-check_result $RC 'iso-read test 1' "$ISO_READ $opts"
-
 exit $RC
 
 #;;; Local Variables: ***
